'' =================================================================================================
''
''   File....... jw_595_ez.spin
''   Purpose.... 74x595 driver
''   Author..... Jon "JonnyMac" Williams
''               Copyright (c) 2009 Jon Williams
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 03 JUL 2009
''   Updated.... 03 JUL 2009 
''
'' =================================================================================================


con

  US_001 = 80_000_000 / 1_000_000
  US_005 = US_001 * 5

  MS_001 = 80_000_000 / 1_000 
  

var

  long  dout
  long  clk
  long  latch


pub init(dpin, cpin, lpin)

'' Sets 74x595 pins to outputs

  dout := dpin
  dira[dout] := 1                                               ' make data an output

  clk := cpin
  outa[clk] := 0                                                ' make clock an output and low
  dira[clk] := 1

  latch := lpin
  outa[latch] := 0                                              ' make latch an output and low
  dira[latch] := 1


pub cleanup

'' Clears 74x595 pins

  dira[dout] := 0  
  dira[clk] := 0
  dira[latch] := 0
  

pub out(b, dolatch)

'' Output byte to 74x595
'' -- optionally latch outputs

  b <<= (32-8)                                                  ' prep for msb output

  ' output bits, MSBFIRST 
  
  repeat 8
    outa[dout] := (b <-= 1) & 1                                 ' output a bit
    outa[clk] := 1                                              ' clock the bit
    waitcnt(US_005 + cnt)
    outa[clk] := 0
    waitcnt(US_005 + cnt)

  if dolatch
    outa[latch] := 1                                            ' latch the outputs
    waitcnt(US_005 + cnt)
    outa[latch] := 0
    

dat

{{

  Copyright (c) 2009 Jon Williams

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   